@@ -18,12 +18,12 @@ def memoize(obj):
         if key not in cache:
             cache[key] = obj(*args, **kwargs)
         return cache[key]
+
     return memoizer
 
 
-class Table(object):
-
-    def __init__(self, name='', df=None):
+class Table:
+    def __init__(self, name="", df=None):
         "Init from a Series/Dataframe (df) of a file (name)"
         if df is not None:  # init from dataframe
             self.df = df
@@ -33,13 +33,32 @@ class Table(object):
             self.df = self.from_name(name).df
             # self.df.name = name
         else:
-            print ('Error: Invalid table name. Valid names are:')
-            print (' '.join(Table.names))
+            print("Error: Invalid table name. Valid names are:")
+            print(" ".join(Table.names))
             return None
 
-    _names = ['AME2003', 'AME2003all', 'AME2012', 'AME2012all', 'AME1995', 'AME1995all',
-             'DUZU', 'FRDM95', 'KTUY05', 'ETFSI12', 'HFB14', 'HFB26', 'TCSM12', 'TCSM13', 'BR2013', 'MAJA88',
-             'GK88', 'WS32010', 'WS32011', 'SVM13']
+    _names = [
+        "AME2003",
+        "AME2003all",
+        "AME2012",
+        "AME2012all",
+        "AME1995",
+        "AME1995all",
+        "DUZU",
+        "FRDM95",
+        "KTUY05",
+        "ETFSI12",
+        "HFB14",
+        "HFB26",
+        "TCSM12",
+        "TCSM13",
+        "BR2013",
+        "MAJA88",
+        "GK88",
+        "WS32010",
+        "WS32011",
+        "SVM13",
+    ]
 
     @classmethod
     def names(cls):
@@ -56,18 +75,20 @@ class Table(object):
     @classmethod
     def from_name(cls, name):
         "Imports a mass table from a file"
-        filename = os.path.join(package_dir, 'data', name + '.txt')
+        filename = os.path.join(package_dir, "data", name + ".txt")
         return cls.from_file(filename, name)
 
     @classmethod
-    def from_file(cls, filename, name=''):
+    def from_file(cls, filename, name=""):
         "Imports a mass table from a file"
-        df = pd.read_csv(filename, header=0, delim_whitespace=True, index_col=[0, 1])['M']
+        df = pd.read_csv(filename, header=0, delim_whitespace=True, index_col=[0, 1])[
+            "M"
+        ]
         df.name = name
         return cls(df=df, name=name)
 
     @classmethod
-    def from_ZNM(cls, Z, N, M, name=''):
+    def from_ZNM(cls, Z, N, M, name=""):
         """
         Creates a table from arrays Z, N and M
 
@@ -84,12 +105,12 @@ class Table(object):
         83  130   -14.45
         Name: Custom Table, dtype: float64
         """
-        df = pd.DataFrame.from_dict({'Z': Z, 'N': N, 'M': M}).set_index(['Z', 'N'])['M']
+        df = pd.DataFrame.from_dict({"Z": Z, "N": N, "M": M}).set_index(["Z", "N"])["M"]
         df.name = name
         return cls(df=df, name=name)
 
     @classmethod
-    def from_array(cls, arr, name=''):
+    def from_array(cls, arr, name=""):
         Z, N, M = arr.T
         return cls.from_ZNM(Z, N, M, name)
 
@@ -107,23 +128,23 @@ class Table(object):
 
         >>> Table('AME2012').tail(10).to_file('last_ten.txt')
         """
-        with open(path, 'w') as f:
-            f.write('Z   N   M\n')
-        self.df.to_csv(path, sep='\t', mode='a')
+        with open(path, "w") as f:
+            f.write("Z   N   M\n")
+        self.df.to_csv(path, sep="\t", mode="a")
 
     @property
     def Z(self):
         """
         Return the proton number Z for all nuclei in the table as a numpy array.
         """
-        return self.df.index.get_level_values('Z').values
+        return self.df.index.get_level_values("Z").values
 
     @property
     def N(self):
         """
         Return the neutron number N for all nuclei in the table as a numpy array.
         """
-        return self.df.index.get_level_values('N').values
+        return self.df.index.get_level_values("N").values
 
     @property
     def A(self):
@@ -150,47 +171,57 @@ class Table(object):
             175  201.43
 
         """
-        if isinstance(index, tuple) and len(index)==2:
-            if isinstance(index[0], int):               # single N: "[82, :]"
+        if isinstance(index, tuple) and len(index) == 2:
+            if isinstance(index[0], int):  # single N: "[82, :]"
                 startZ, stopZ = index[0], index[0]
 
             if isinstance(index[1], int):
-                startN, stopN = index[1], index[1]      # single N: "[:, 126]"
+                startN, stopN = index[1], index[1]  # single N: "[:, 126]"
 
-            if isinstance(index[0], slice):             # Z slice: "[:, 126]"
+            if isinstance(index[0], slice):  # Z slice: "[:, 126]"
                 startZ, stopZ, stepZ = index[0].start, index[0].stop, index[0].step
 
-            if isinstance(index[1], slice):              # N slice: "[:, 126]"
+            if isinstance(index[1], slice):  # N slice: "[:, 126]"
                 startN, stopN, stepN = index[1].start, index[1].stop, index[1].step
 
-            if not startZ: startZ = self.Z.min()  # might be optimized
-            if not stopZ:  stopZ = self.Z.max()
-            if not startN: startN = self.N.min()
-            if not stopN:  stopN = self.N.max()
+            if not startZ:
+                startZ = self.Z.min()  # might be optimized
+            if not stopZ:
+                stopZ = self.Z.max()
+            if not startN:
+                startN = self.N.min()
+            if not stopN:
+                stopN = self.N.max()
 
             x = self.df.reset_index()
-            x = x.loc[(x.Z>=startZ)&(x.Z<=stopZ)&(x.N>=startN)&(x.N<=stopN)]
-            df = x.set_index(['Z', 'N']).sortlevel(0)
+            x = x.loc[
+                (x.Z >= startZ) & (x.Z <= stopZ) & (x.N >= startN) & (x.N <= stopN)
+            ]
+            df = x.set_index(["Z", "N"]).sort_index(0)
             return Table(df=df[df.columns[0]], name=self.name)
 
     def __setitem__(self, key, value):
         Z = key[0]
         N = key[1]
-        self.df.ix[(Z,N)] = value
+        self.df.loc[(Z, N)] = value
 
     def __getattr__(self, attr):
         # TODO: Pythonize
         "Pass properties and method calls to the DataFrame object"
         instance_method = getattr(self.df, attr)
         if callable(instance_method):
+
             def fn(*args, **kwargs):
-                result = instance_method(*args, **kwargs)   # ()->call the instance method
+                result = instance_method(
+                    *args, **kwargs
+                )  # ()->call the instance method
                 if isinstance(result, (pd.DataFrame, pd.Series)):
                     try:
                         name = result.name
                     except AttributeError:
                         name = None
                     return Table(name=name, df=result)  # wrap in Table class
+
             return fn
         else:
             return instance_method
@@ -200,22 +231,19 @@ class Table(object):
             yield e
 
     def __add__(self, other):
-        return Table(df=self.df + other.df,
-                     name="{}+{}".format(self.name, other.name))
+        return Table(df=self.df + other.df, name="{}+{}".format(self.name, other.name))
 
     def __sub__(self, other):
-        return Table(df=self.df - other.df,
-                     name="{}+{}".format(self.name, other.name))
+        return Table(df=self.df - other.df, name="{}+{}".format(self.name, other.name))
 
     def __div__(self, other):
-        return Table(df=self.df - other.df,
-                     name="{}+{}".format(self.name, other.name))
+        return Table(df=self.df - other.df, name="{}+{}".format(self.name, other.name))
 
     def align(self, *args, **kwargs):
         result = self.df.align(*args, **kwargs)[0]
         return Table(result.name, result)
 
-    def select(self, condition, name=''):
+    def select(self, condition, name=""):
         """
         Selects nuclei according to a condition on Z,N or M
 
@@ -233,14 +261,14 @@ class Table(object):
         >>> A_gt_160 = lambda Z,N: Z + N > 160
         >>> Table('AME2003').select(A_gt_160)
         """
-        if condition.func_code.co_argcount == 1:
+        if condition.__code__.co_argcount == 1:
             idx = [(Z, N) for (Z, N), M in self if condition(M)]
-        if condition.func_code.co_argcount == 2:
+        if condition.__code__.co_argcount == 2:
             idx = [(Z, N) for (Z, N) in self.index if condition(Z, N)]
-        if condition.func_code.co_argcount == 3:
+        if condition.__code__.co_argcount == 3:
             idx = [(Z, N) for (Z, N), M in self if condition(Z, N, M)]
-        index = pd.MultiIndex.from_tuples(idx, names=['Z', 'N'])
-        return Table(df=self.df.ix[index], name=name)
+        index = pd.MultiIndex.from_tuples(idx, names=["Z", "N"])
+        return Table(df=self.df.loc[index], name=name)
 
     def at(self, nuclei):
         """Return a selection of the Table at positions given by ``nuclei``
@@ -262,11 +290,11 @@ class Table(object):
             82     1102.876416
         82  126    1636.486450
         """
-        index = pd.MultiIndex.from_tuples(nuclei, names=['Z', 'N'])
-        return Table(df=self.df.ix[index], name=self.name)
+        index = pd.MultiIndex.from_tuples(nuclei, names=["Z", "N"])
+        return Table(df=self.df.loc[index], name=self.name)
 
     @classmethod
-    def empty(cls, name=''):
+    def empty(cls, name=""):
         return cls(df=pd.DataFrame(index=[], columns=[]), name=name)
 
     def __len__(self):
@@ -303,7 +331,7 @@ class Table(object):
         ----------
         Table('AME2003').intersection(Table('AME1995'))
         """
-        idx = self.df.index & table.df.index
+        idx = self.df.index.intersection(table.df.index)
         return Table(df=self.df[idx], name=self.name)
 
     def not_in(self, table):
@@ -321,7 +349,7 @@ class Table(object):
         >>> Table('AME2003').not_in(Table('AME1995'))[8:,8:].count
         389
         """
-        idx = self.df.index - table.df.index
+        idx = self.df.index.difference(table.df.index)
         return Table(df=self.df[idx], name=self.name)
 
     @property
@@ -346,7 +374,7 @@ class Table(object):
         """
         Selects odd-even nuclei from the table
         """
-        return self.select(lambda Z, N: (Z % 2) and not(N % 2), name=self.name)
+        return self.select(lambda Z, N: (Z % 2) and not (N % 2), name=self.name)
 
     @property
     @memoize
@@ -354,7 +382,7 @@ class Table(object):
         """
         Selects even-odd nuclei from the table
         """
-        return self.select(lambda Z, N: not(Z % 2) and (N % 2), name=self.name)
+        return self.select(lambda Z, N: not (Z % 2) and (N % 2), name=self.name)
 
     @property
     @memoize
@@ -362,9 +390,9 @@ class Table(object):
         """
         Selects even-even nuclei from the table
         """
-        return self.select(lambda Z, N: not(Z % 2) and not(N % 2), name=self.name)
+        return self.select(lambda Z, N: not (Z % 2) and not (N % 2), name=self.name)
 
-    def error(self, relative_to='AME2003'):
+    def error(self, relative_to="AME2003"):
         """
         Calculate error difference
 
@@ -380,7 +408,7 @@ class Table(object):
         df = self.df - Table(relative_to).df
         return Table(df=df)
 
-    def rmse(self, relative_to='AME2003'):
+    def rmse(self, relative_to="AME2003"):
         """Calculate root mean squared error
 
         Parameters
@@ -428,61 +456,64 @@ class Table(object):
         AMU = 931.494028
         # MeV
         df = self.Z * (M_P + M_E) + (self.A - self.Z) * M_N - (self.df + self.A * AMU)
-        return Table(df=df, name='BE' + '(' + self.name + ')')
+        return Table(df=df, name="BE" + "(" + self.name + ")")
 
     @property
     @memoize
     def q_alpha(self):
         """Return Q_alpha"""
-        M_ALPHA = 2.4249156         # He4 mass excess in MeV
+        M_ALPHA = 2.4249156  # He4 mass excess in MeV
         f = lambda parent, daugther: parent - daugther - M_ALPHA
-        return self.derived('Q_alpha', (-2, -2), f)
+        return self.derived("Q_alpha", (-2, -2), f)
 
     @property
     @memoize
     def q_beta(self):
         """Return Q_beta"""
         f = lambda parent, daugther: parent - daugther
-        return self.derived('Q_beta', (1, -1), f)
+        return self.derived("Q_beta", (1, -1), f)
 
     @property
     @memoize
     def s2n(self):
         """Return 2 neutron separation energy"""
-        M_N = 8.0713171         # neutron mass excess in MeV
+        M_N = 8.0713171  # neutron mass excess in MeV
         f = lambda parent, daugther: -parent + daugther + 2 * M_N
-        return self.derived('s2n', (0, -2), f)
+        return self.derived("s2n", (0, -2), f)
 
     @property
     @memoize
     def s1n(self):
         """Return 1 neutron separation energy"""
-        M_N = 8.0713171         # neutron mass excess in MeV
+        M_N = 8.0713171  # neutron mass excess in MeV
         f = lambda parent, daugther: -parent + daugther + M_N
-        return self.derived('s1n', (0, -1), f)
+        return self.derived("s1n", (0, -1), f)
 
     @property
     @memoize
     def s2p(self):
         """Return 2 proton separation energy"""
-        M_P = 7.28897050         # proton mass excess in MeV
+        M_P = 7.28897050  # proton mass excess in MeV
         f = lambda parent, daugther: -parent + daugther + 2 * M_P
-        return self.derived('s2p', (-2, 0), f)
+        return self.derived("s2p", (-2, 0), f)
 
     @property
     @memoize
     def s1p(self):
         """Return 1 proton separation energy"""
-        M_P = 7.28897050         # proton mass excess in MeV
+        M_P = 7.28897050  # proton mass excess in MeV
         f = lambda parent, daugther: -parent + daugther + M_P
-        return self.derived('s1p', (-1, 0), f)
+        return self.derived("s1p", (-1, 0), f)
 
     def derived(self, name, relative_coords, formula):
         """Helper function for derived quantities"""
-        relZ, relN = relative_coords
-        daughter_idx = [(x[0] + relZ, x[1] + relN) for x in self.df.index]
-        values = formula(self.df.values, self.df.loc[daughter_idx].values)
-        return Table(df=pd.Series(values, index=self.df.index, name=name + '(' + self.name + ')'))
+        dZ, dN = relative_coords
+        daughter_idx = [(Z + dZ, N + dN) for Z, N in self.df.index]
+        idx = self.df.index.intersection(daughter_idx)
+        values = formula(self.df.values, self.df.reindex(daughter_idx).values)
+        return Table(
+            df=pd.Series(values, index=self.df.index, name=name + "(" + self.name + ")")
+        )
 
     @property
     @memoize
@@ -493,7 +524,11 @@ class Table(object):
         """
         idx = [(x[0] + 0, x[1] + 2) for x in self.df.index]
         values = self.s2n.values - self.s2n.loc[idx].values
-        return Table(df=pd.Series(values, index=self.df.index, name='ds2n' + '(' + self.name + ')'))
+        return Table(
+            df=pd.Series(
+                values, index=self.df.index, name="ds2n" + "(" + self.name + ")"
+            )
+        )
 
     @property
     @memoize
@@ -504,7 +539,11 @@ class Table(object):
         """
         idx = [(x[0] + 2, x[1]) for x in self.df.index]
         values = self.s2p.values - self.s2p.loc[idx].values
-        return Table(df=pd.Series(values, index=self.df.index, name='ds2p' + '(' + self.name + ')'))
+        return Table(
+            df=pd.Series(
+                values, index=self.df.index, name="ds2p" + "(" + self.name + ")"
+            )
+        )
 
     def __repr__(self):
         return self.df.__repr__()
@@ -512,12 +551,19 @@ class Table(object):
     def __str__(self):
         return self.df.__str__()
 
-    def join(self, join='outer', *tables):
+    def join(self, join="outer", *tables):
         return Table(df=pd.concat([self.df] + [table.df for table in tables], axis=1))
 
-
-    def chart_plot(self, ax=None, cmap='RdBu',
-               xlabel='N', ylabel='Z', grid_on=True, colorbar=True):
+    def chart_plot(
+        self,
+        ax=None,
+        cmap="RdBu",
+        xlabel="N",
+        ylabel="Z",
+        grid_on=True,
+        colorbar=True,
+        save_path=None,
+    ):
         """Plot a nuclear chart with (N,Z) as axis and the values
         of the Table as a color scale
 
@@ -547,7 +593,7 @@ class Table(object):
         >>> Table('FRDM95').error().chart_plot()
 
         """
-        from matplotlib.mlab import griddata
+        from scipy.interpolate import griddata
         from numpy import linspace, meshgrid
         import matplotlib.pyplot as plt
 
@@ -556,31 +602,37 @@ class Table(object):
         y = self.dropna().Z
         z = self.dropna().values
 
-        #convert to matplotlibs grid format
+        # convert to matplotlibs grid format
         xi = linspace(min(x), max(x), max(x) - min(x) + 1)
         yi = linspace(min(y), max(y), max(y) - min(y) + 1)
-        Z = griddata(x, y, z, xi, yi)
         X, Y = meshgrid(xi, yi)
 
+        Z = griddata((x, y), z, (X, Y), method="linear")
+
         # create and customize plot
         if ax is None:
             ax = plt.gca()
-        chart = ax.pcolormesh(X, Y, Z, cmap=cmap)
+        chart = ax.pcolormesh(X, Y, Z, cmap=cmap, shading="auto")
         ax.set_xlabel(xlabel)
         ax.set_ylabel(ylabel)
         ax.grid(grid_on)
-        ax.set_aspect('equal')
+        ax.set_aspect("equal")
         if colorbar:
             plt.colorbar(chart)
-
+        if save_path is not None:
+            fig = plt.gcf()
+            fig.savefig(save_path)
         return ax
 
-if __name__ == '__main__':
-    # print Table('AME2003').head().join(Table('AME2012').head(), Table('DUZU').odd_odd.head())
-    # print Table.from_file('data/AME1995.txt', 'test')
-    # print Table.load('AME1995')
-    # print Table('GK88')
-    # print Table('FRDM95').
-    print Table('FRDM95')[101, 220]
-    # print Table('AME2012').tail()
-    # print Table.from_list([1,1])
+    # def chart_altair(self):
+    #     import altair as alt
+
+    #     data = self.df.dropna().reset_index()
+    #     data["color"] = self.df.dropna().values
+
+    #     chart = (
+    #         alt.Chart(data)
+    #         .mark_rect()
+    #         .encode(alt.X("N"), alt.Y("Z"), alt.Color("color"))
+    #     )
+    #     chart.save("chart.html")
